#include "stdafx.h"
#include "VideoDialog.h"
#include "wa_ipc.h"
#include "WAEmuWnd.h"
#include "MainWnd.h"
#include "MainHelper.h"

#include <assert.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int infoWidth;
int infoHeight;
CString infoTitle;

CVideoDialog::CVideoDialog() : CDialog(CVideoDialog::IDD), m_bMouseDown(false), m_w(10), m_h(10), m_fullScreen(false)
{
	TRACE(_T("CVideoDialog"));
	InitializeCriticalSection(&m_cs);
	menu.LoadMenu(IDR_MENUVIDEO);

	m_pDD = NULL;
	m_pddsFrontBuffer = NULL;
	m_pddsBackBuffer = NULL;
}

CVideoDialog::~CVideoDialog()
{
	SAFE_RELEASE(m_pddsFrontBuffer);
	SAFE_RELEASE(m_pddsBackBuffer);

	if (m_pDD)
		m_pDD->SetCooperativeLevel(m_hWnd, DDSCL_NORMAL);

	SAFE_RELEASE(m_pDD);
	menu.DestroyMenu();

	DestroyWindow();
	DeleteCriticalSection(&m_cs);
}

IMPLEMENT_MENUXP(CVideoDialog, CDialog)

BEGIN_MESSAGE_MAP(CVideoDialog, CDialog)
ON_MENUXP_MESSAGES()
ON_WM_KEYDOWN()
ON_WM_PAINT()
ON_WM_SHOWWINDOW()
ON_WM_MOUSEWHEEL()
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_RBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_CREATE()
ON_COMMAND(ID_PLAY_VIDEO, OnPlay)
ON_COMMAND(ID_PAUSE_VIDEO, OnPause)
ON_COMMAND(ID_STOP_VIDEO, OnStop)
ON_COMMAND(ID_PREVIOUS_VIDEO, OnPrevious)
ON_COMMAND(ID_NEXT_VIDEO, OnNext)
ON_COMMAND(ID_ZOOM25, OnZoom25)
ON_COMMAND(ID_ZOOM50, OnZoom50)
ON_COMMAND(ID_ZOOM100, OnZoom100)
ON_COMMAND(ID_ZOOM200, OnZoom200)
ON_COMMAND(ID_FULLSCREEN, OnFullScreen)
ON_COMMAND(ID_CLOSEVIDEO, OnCloseVideo)
ON_MESSAGE(WM_DRAW_IMAGE, Restore)
END_MESSAGE_MAP()

void CVideoDialog::SetInfo(int w, int h, int vflip, double aspectratio, unsigned int fmt)
{
	m_w = w;
	m_h = h;
	m_vflip = vflip;
	m_fmt = fmt;
	if(m_fullScreen)
		SetWindowPos(&wndTopMost, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), SWP_NOZORDER);
	else
		SetWindowPos(&wndNoTopMost, pMainFrame->VideoX, pMainFrame->VideoY, w, h, SWP_NOZORDER);
}
void CVideoDialog::DoDataExchange(CDataExchange* pDX)
{
	CMenuXP::SetXPLookNFeel(this, pMainFrame->XPLookNFeel);
	CDialog::DoDataExchange(pDX);
}

CVideoDialog* CVideoDialog::getInstance()
{
	if (VideoDialog != NULL && VideoDialog->m_hWnd != NULL)
		VideoDialog->DestroyWindow();
	if (VideoDialog != NULL && VideoDialog->m_hWnd == NULL)
	{
		delete VideoDialog;
		VideoDialog = NULL;
	}
	if (VideoDialog == NULL)
	{
		VideoDialog = new CVideoDialog();
		VideoDialog->Create(CVideoDialog::IDD);
	}
	else
		VideoDialog->SetFocus();
	return VideoDialog;
}

BOOL CVideoDialog::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	HRESULT hr;

	// DDraw stuff begins here
	if (FAILED(hr = DirectDrawCreateEx(NULL, (VOID * *)&m_pDD, IID_IDirectDraw7, NULL)))
		return E_FAIL;
	
	// Set cooperative level
	hr = m_pDD->SetCooperativeLevel(m_hWnd, DDSCL_NORMAL);
	if (FAILED(hr))
		return E_FAIL;
	
	LPDIRECTDRAWCLIPPER pcClipper;

	// Create the primary surface
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	if (FAILED(m_pDD->CreateSurface(&ddsd, &m_pddsFrontBuffer, NULL)))
		return E_FAIL;

	if (FAILED(hr = m_pDD->CreateClipper(0, &pcClipper, NULL)))
		return E_FAIL;

	if (FAILED(hr = pcClipper->SetHWnd(0, m_hWnd)))
	{
		pcClipper->Release();
		return E_FAIL;
	}

	if (FAILED(hr = m_pddsFrontBuffer->SetClipper(pcClipper)))
	{
		pcClipper->Release();
		return E_FAIL;
	}

	// Done with clipper
	pcClipper->Release();

	CRect rect1, rect2;

	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rect1, 0);
	GetWindowRect(&rect2);
	if (pMainFrame->VideoW == -1000)
		pMainFrame->VideoW = rect2.Width();
	if (pMainFrame->VideoH == -1000)
		pMainFrame->VideoH = rect2.Height();
	if (pMainFrame->VideoX == -1000)
		pMainFrame->VideoX = (rect1.right - pMainFrame->VideoW) / 2;
	if (pMainFrame->VideoY == -1000)
		pMainFrame->VideoY = (rect1.bottom - pMainFrame->VideoH) / 2;
	if (pMainFrame->VideoX < 0)
		pMainFrame->VideoX = 0;
	if (pMainFrame->VideoY < 0)
		pMainFrame->VideoY = 0;
	if (pMainFrame->VideoX + pMainFrame->VideoW > rect1.right)
		pMainFrame->VideoX = rect1.right - pMainFrame->VideoW;
	if (pMainFrame->VideoY + pMainFrame->VideoH > rect1.bottom)
		pMainFrame->VideoY = rect1.bottom - pMainFrame->VideoH;
	m_fullScreen = pMainFrame->FullScreen;
	OnFullScreen();

	return TRUE;
}

void CVideoDialog::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	CDialog::OnKeyDown(nChar, nRepCnt, nFlags);
	switch (nChar)
	{
	case VK_F1:
		m_fullScreen = false;
		SetWindowPos(&wndNoTopMost, pMainFrame->VideoX, pMainFrame->VideoY, pMainFrame->VideoW, pMainFrame->VideoH, SWP_NOZORDER);
		return;
	case VK_F2:
		m_fullScreen = true;
		SaveWindowPos();
		SetWindowPos(&wndTopMost, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), SWP_NOZORDER);
		break;
	}
}


void CVideoDialog::OnPaint()
{
	SendMessage(WM_DRAW_IMAGE);

	CDialog::OnPaint();
}

void CVideoDialog::OnShowWindow(BOOL bShow, UINT nStatus)
{
	EnterCriticalSection(&m_cs);
	CDialog::OnShowWindow(bShow, nStatus);
	LeaveCriticalSection(&m_cs);
}

void CVideoDialog::paintFrame(void* frame)
{
	if (m_pDD == NULL)
		return;		

	EnterCriticalSection(&m_cs);
	HRESULT hr;
	if(IsWindowVisible()==TRUE)
	{
		SAFE_RELEASE(m_pddsBackBuffer);
			
		DDSURFACEDESC2 ddsd;
		ZeroMemory(&ddsd, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		// Create the backbuffer surface
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
		ddsd.dwWidth = m_w;
		ddsd.dwHeight = m_h;

		ddsd.ddpfPixelFormat.dwSize = sizeof(ddsd.ddpfPixelFormat);
		ddsd.ddpfPixelFormat.dwFlags = DDPF_FOURCC | DDPF_YUV;
		ddsd.ddpfPixelFormat.dwFourCC = m_fmt;
		ddsd.ddpfPixelFormat.dwYUVBitCount = 8;

		if (!FAILED(hr = m_pDD->CreateSurface(&ddsd, &m_pddsBackBuffer, NULL)))
		{
			if (!FAILED(hr = m_pddsBackBuffer->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL)))
			{
				unsigned char* backbuffervideodata = (unsigned char*)ddsd.lpSurface;

				YV12_PLANES* fr = (YV12_PLANES*)frame;

				unsigned char* lpSurfy = backbuffervideodata;
				unsigned char* lpSurfu = backbuffervideodata + (5 * ddsd.lPitch* m_h) / 4;
				unsigned char* lpSurfv = backbuffervideodata + ddsd.lPitch* m_h;
				unsigned char* ptry = fr->y.baseAddr;
				unsigned char* ptrv = fr->v.baseAddr;
				unsigned char* ptru = fr->u.baseAddr;

				for (long y = 0; y < m_h; y++)
				{
					// Y
					memcpy(lpSurfy, ptry, fr->y.rowBytes);
					ptry += fr->y.rowBytes;
					lpSurfy += ddsd.lPitch;

					if (y % 2 == 0)
					{
						// V
						memcpy(lpSurfv, ptrv, fr->v.rowBytes);
						ptrv += fr->v.rowBytes;
						lpSurfv += ddsd.lPitch / 2;
						// U
						memcpy(lpSurfu, ptru, fr->u.rowBytes);
						ptru += fr->u.rowBytes;
						lpSurfu += ddsd.lPitch / 2;
					}
				}
				m_pddsBackBuffer->Unlock(NULL);
			}
		}
	}
	LeaveCriticalSection(&m_cs);
	SendMessage(WM_DRAW_IMAGE);
}

#define MAX(x,y) (x>0?(x>y?x:y):(x<y?x:y))

BOOL CVideoDialog::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	if(!m_fullScreen)
	{
		int x, y;
		int dx = MAX((zDelta * m_w / m_h), 3 / 2);//must be > 4./3.

		CRect rect;
		GetClientRect(&rect);

		x = rect.Width() + dx;
		y = x * m_h / m_w;
		SetWindowPos(NULL, 0, 0, x, y, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
	}
    
	return CDialog::OnMouseWheel(nFlags, zDelta, pt);
}

void CVideoDialog::OnLButtonDown(UINT nFlags, CPoint point)
{
	if (!m_bMouseDown)
	{
		m_mouseDownPoint = point;
		CRect rect;
		GetWindowRect(&rect);

		m_mouseDownPointOrigine.x = rect.left;
		m_mouseDownPointOrigine.y = rect.top;
	}
	m_bMouseDown = true;
	CDialog::OnLButtonDown(nFlags, point);
}

void CVideoDialog::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_bMouseDown = false;
	ReleaseCapture();
	m_nRep2 = 0;
	CDialog::OnLButtonUp(nFlags, point);
}

void CVideoDialog::OnRButtonUp(UINT nFlags, CPoint point)
{
	CMenu* pPopup;
 	pPopup = menu.GetSubMenu(0);
	m_Helper.translateMenu(pPopup, IDR_MENUVIDEO);

	pPopup->CheckMenuItem(ID_FULLSCREEN, MF_BYCOMMAND | (m_fullScreen? MF_CHECKED : MF_UNCHECKED));
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
}

void CVideoDialog::OnMouseMove(UINT nFlags, CPoint point)
{
	m_nRep2++;
	if ((m_bMouseDown) && (m_nRep2 % 2 == 0) && !m_fullScreen)
	{
		CRect rect;
		GetWindowRect(&rect);

		SetWindowPos(NULL, rect.left + point.x - m_mouseDownPoint.x, rect.top + point.y - m_mouseDownPoint.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
	}
	CDialog::OnMouseMove(nFlags, point);
}

LRESULT CVideoDialog::Restore(WPARAM wParam, LPARAM lParam)
{
	if (NULL == m_pddsFrontBuffer || m_pddsBackBuffer == NULL)
	{
		return E_POINTER;
	}

	EnterCriticalSection(&m_cs);
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	m_pddsBackBuffer->GetSurfaceDesc(&ddsd);
	CRect rect;
	CRect m_rcWindow;
	GetWindowRect(&m_rcWindow);
	GetWindowRect(&rect);
	if(ddsd.dwWidth * rect.Height() < ddsd.dwHeight * rect.Width())
		rect.right = rect.left + rect.Height() * ddsd.dwWidth / ddsd.dwHeight;
	else
		rect.bottom = rect.top + rect.Width() * ddsd.dwHeight / ddsd.dwWidth;
	int x = (m_rcWindow.right - m_rcWindow.left - rect.Width()) /2;
	int y = (m_rcWindow.bottom - m_rcWindow.top - rect.Height()) /2;
	rect.OffsetRect(x, y);
	CRect rect1(m_rcWindow), rect2(m_rcWindow);
	if(m_rcWindow.left == rect.left)
	{
		rect1.bottom = rect.top;
		rect2.top = rect.bottom;
	}
	else
	{
		rect1.right = rect.left;
		rect2.left = rect.right;
	}
    DDBLTFX ddbltfx;
    ZeroMemory( &ddbltfx, sizeof(ddbltfx) );
    ddbltfx.dwSize      = sizeof(ddbltfx);
    ddbltfx.dwFillColor = RGB(0, 0, 0);

    if(x!= 0 || y != 0)
	{
		m_pddsFrontBuffer->Blt( &rect1, NULL, NULL, DDBLT_COLORFILL, &ddbltfx );
		m_pddsFrontBuffer->Blt( &rect2, NULL, NULL, DDBLT_COLORFILL, &ddbltfx );
	}
	m_pddsFrontBuffer->Blt(&rect, m_pddsBackBuffer, NULL, DDBLT_WAIT, NULL);
	LeaveCriticalSection(&m_cs);
	return TRUE;
}

void CVideoDialog::SaveWindowPos()
{
	CRect rect;
	GetWindowRect(&rect);
	pMainFrame->VideoX = rect.left;
	pMainFrame->VideoY = rect.top;
	pMainFrame->VideoW = rect.Width();
	pMainFrame->VideoH = rect.Height();
	pMainFrame->FullScreen = m_fullScreen;
}

void CVideoDialog::SaveWindowPosAndDestroy()
{
	ShowWindow(SW_RESTORE);
	SaveWindowPos();

	ShowWindow(SW_HIDE);
	DestroyWindow();
}

BOOL CVideoDialog::isFullScreen()
{
	return m_fullScreen;
}

void CVideoDialog::OnPlay()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON2, 0);
}

void CVideoDialog::OnPause()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
}

void CVideoDialog::OnStop()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
}

void CVideoDialog::OnPrevious()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON1, 0);
}

void CVideoDialog::OnNext()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON5, 0);
}

void CVideoDialog::OnZoom25()
{
	m_fullScreen = false;
	SetWindowPos(&wndNoTopMost, 0, 0, m_w / 4, m_h / 4, SWP_NOMOVE | SWP_NOZORDER);
	SaveWindowPos();
}

void CVideoDialog::OnZoom50()
{
	m_fullScreen = false;
	SetWindowPos(&wndNoTopMost, 0, 0, m_w / 2, m_h / 2, SWP_NOMOVE | SWP_NOZORDER);
	SaveWindowPos();
}

void CVideoDialog::OnZoom100()
{
	m_fullScreen = false;
	SetWindowPos(&wndNoTopMost, 0, 0, m_w, m_h, SWP_NOMOVE | SWP_NOZORDER);
	SaveWindowPos();
}

void CVideoDialog::OnZoom200()
{
	m_fullScreen = false;
	SetWindowPos(&wndNoTopMost, 0, 0, 2 * m_w, 2 * m_h, SWP_NOMOVE | SWP_NOZORDER);
	SaveWindowPos();
}

void CVideoDialog::OnFullScreen()
{
	if(!m_fullScreen)
	{
		m_fullScreen = true;
		SaveWindowPos();
		SetWindowPos(&wndTopMost, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), SWP_NOZORDER);
	}
	else
	{
		m_fullScreen = false;
		SetWindowPos(&wndNoTopMost, pMainFrame->VideoX, pMainFrame->VideoY, pMainFrame->VideoW, pMainFrame->VideoH, SWP_NOZORDER);
	}
}

void CVideoDialog::OnCloseVideo()
{
	ShowWindow(SW_HIDE);
}

CIVideoOutput::CIVideoOutput()
{
}

CIVideoOutput::~CIVideoOutput()
{
	if (VideoDialog != NULL && VideoDialog->m_hWnd != NULL)
		VideoDialog->DestroyWindow();
	delete VideoDialog;
	VideoDialog = NULL;
}

int CIVideoOutput::open(int w, int h, int vflip, double aspectratio, unsigned int fmt)
{
	char ch[5];
	ch[0] = (char)(fmt & 0xFF);
	ch[1] = (char)((fmt >> 8) & 0xFF);
	ch[2] = (char)((fmt >> 16) & 0xFF);
	ch[3] = (char)((fmt >> 24) & 0xFF);
	ch[4] = 0;
	TRACE(_T("open w=%d h=%d vflip=%d aspectratio=%f %c%c%c%c\n"), w, h, vflip, aspectratio, ch[0], ch[1], ch[2], ch[3]);

	infoWidth = w;
	infoHeight = h;
	VideoDialog->ShowWindow(SW_SHOW);
	VideoDialog->SetInfo(w, h, vflip, aspectratio, fmt);
	return 0;
}

void CIVideoOutput::setcallback(LRESULT(*msgcallback)(void* token, HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam), void* token)
{
	TRACE(_T("setcallback\n"));
}

void CIVideoOutput::close()
{
	TRACE(_T("close\n"));
	VideoDialog->ShowWindow(SW_HIDE);
}

void CIVideoOutput::draw(void* frame)
{
	VideoDialog->paintFrame(frame);
}

void CIVideoOutput::drawSubtitle(SubsItem* item)
{
	TRACE(_T("drawSubtitle\n"));
}

void CIVideoOutput::showStatusMsg(const char* text)
{
	TRACE(_T("showStatusMsg %s\n"), text);
}

int CIVideoOutput::get_latency()
{
	TRACE(_T("get_latency\n"));
	return 0;
}

void CIVideoOutput::notifyBufferState(int bufferstate)
{
	TRACE(_T("notifyBufferState %d\n"), bufferstate);
} /* 0-255*/

int CIVideoOutput::extended(int param1, int param2, int param3)
{
	switch (param1)
	{
	case VIDUSER_SET_INFOSTRING:
		//TRACE(_T("extended VIDUSER_SET_INFOSTRING %d %d\n"), param2, param3);
		infoTitle = _T("");
		infoTitle+= (char *)param2;
		//TRACE(_T("extended VIDUSER_SET_INFOSTRING %s %d\n"), infoTitle, param3);
		break;
	case VIDUSER_GET_VIDEOHWND:
		TRACE(_T("extended VIDUSER_GET_VIDEOHWND %d %d\n"), param2, param3);
		break;
	case VIDUSER_SET_VFLIP:
		TRACE(_T("extended VIDUSER_SET_VFLIP %d %d\n"), param2, param3);
		break;
	case VIDUSER_SET_TRACKSELINTERFACE:
		TRACE(_T("extended VIDUSER_SET_TRACKSELINTERFACE %d %d\n"), param2, param3);
		break;
	}
	return 0;
} //Dispatchable, eat this!
