#ifndef VIDEODIALOG_H
#define VIDEODIALOG_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Tools/MenuXP.h"
#include <ddraw.h>
#include <mmsystem.h>
#include <wa_ipc.h>
#include "wa_msgids.h"
#include "resource.h"

#define SAFE_DELETE(p) { if(p) { delete (p); (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

#define WM_DRAW_IMAGE WM_USER+1

class CVideoDialog : public CDialog
{
protected:
	LPDIRECTDRAW7 m_pDD;
	LPDIRECTDRAWSURFACE7 m_pddsFrontBuffer;
	LPDIRECTDRAWSURFACE7 m_pddsBackBuffer;
public:
	CRITICAL_SECTION m_cs;
	CVideoDialog();
	~CVideoDialog();
	CVideoDialog* getInstance();

	enum
	{
		IDD = IDD_VIDEO
	};

	void paintFrame(void* frame);
	void SetInfo(int w, int h, int vflip, double aspectratio, unsigned int fmt);
	// Display methods
	LRESULT Restore(WPARAM wParam, LPARAM lParam);
	void SaveWindowPos();
	void SaveWindowPosAndDestroy();
	BOOL isFullScreen();
private:
	int m_w;
	int m_h; 
	int m_vflip; 
	unsigned int m_fmt;
    int m_nRep2;
	bool m_bMouseDown;
    CPoint m_mouseDownPoint;
    CPoint m_mouseDownPointOrigine;
	BOOL m_fullScreen;
	CMenu menu;
protected:
	virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
	virtual BOOL OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnPaint();
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnPlay();
	afx_msg void OnPause();
	afx_msg void OnStop();
	afx_msg void OnPrevious();
	afx_msg void OnNext();
	afx_msg void OnZoom25();
	afx_msg void OnZoom50();
	afx_msg void OnZoom100();
	afx_msg void OnZoom200();
	afx_msg void OnFullScreen();
	afx_msg void OnCloseVideo();
	DECLARE_MESSAGE_MAP()
		DECLARE_MENUXP()
};

class CIVideoOutput : public IVideoOutput
{
public:
	CIVideoOutput();
	~CIVideoOutput();
	int open(int w, int h, int vflip, double aspectratio, unsigned int fmt);
	void setcallback(LRESULT(*msgcallback)(void* token, HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam), void* token);
	void close();
	void draw(void* frame);
	void drawSubtitle(SubsItem* item);
	void showStatusMsg(const char* text);
	int get_latency();
	void notifyBufferState(int bufferstate);
	int extended(int param1, int param2, int param3); // Dispatchable, eat this!
};
#endif
extern int infoWidth;
extern int infoHeight;
extern CString infoTitle;
extern CIVideoOutput* iVidOut;
extern CVideoDialog* VideoDialog;