#include "stdafx.h"
#include "Volume.h"
#include "MainWnd.h"
#include <Mmsystem.h>

#define BAD_DWORD     (DWORD)0

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVolume::CVolume() : m_bAvailable(false), m_uMixerID(0L), m_dwMixerHandle(0L), m_dwMinimalVolume(BAD_DWORD), m_dwMaximalVolume(BAD_DWORD)
{
}

void CVolume::Init()
{
	if (mixerGetNumDevs() < 1)
		return;
	MMRESULT mmResult;
	mmResult = mixerOpen((LPHMIXER)&m_dwMixerHandle, 0, (DWORD)pMainFrame->m_hWnd, 0L, CALLBACK_WINDOW);
	if (mmResult != MMSYSERR_NOERROR)
		return;
	MIXERLINE MixerLine;
	MixerLine.cbStruct = sizeof(MIXERLINE);
	MixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
	mmResult = mixerGetLineInfo((HMIXEROBJ)m_dwMixerHandle, &MixerLine, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE);
	if (mmResult != MMSYSERR_NOERROR)
		return;

	MIXERCONTROL Control;
	Control.cbStruct = sizeof(MIXERCONTROL);

	MIXERLINECONTROLS LineControls;
	LineControls.cbStruct = sizeof(MIXERLINECONTROLS);

	LineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	LineControls.dwLineID = MixerLine.dwLineID;
	LineControls.cControls = 1;
	LineControls.cbmxctrl = sizeof(MIXERCONTROL);
	LineControls.pamxctrl = &Control;
	mmResult = mixerGetLineControls((HMIXEROBJ)m_dwMixerHandle, &LineControls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE);
	if (mmResult == MMSYSERR_NOERROR)
	{
		if (!(Control.fdwControl & MIXERCONTROL_CONTROLF_DISABLED))
			m_bAvailable = true;
	}

	m_nChannelCount = MixerLine.cChannels;
	m_dwLineID = LineControls.dwLineID;
	m_dwVolumeControlID = Control.dwControlID;
	m_dwMinimalVolume = Control.Bounds.dwMinimum;
	m_dwMaximalVolume = Control.Bounds.dwMaximum;
	m_dwVolumeStep = Control.Metrics.cSteps;
}

CVolume::~CVolume()
{
	if (m_dwMixerHandle)
		mixerClose((HMIXER)m_dwMixerHandle);
}

DWORD CVolume::GetCurrentVolume()
{
	if (!m_bAvailable)
		return BAD_DWORD;

	MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
	if (!aDetails)
		return BAD_DWORD;
	MIXERCONTROLDETAILS ControlDetails;
	memset(&ControlDetails, 0, sizeof(MIXERCONTROLDETAILS));
	ControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
	ControlDetails.dwControlID = m_dwVolumeControlID;
	ControlDetails.cChannels = m_nChannelCount;
	ControlDetails.cMultipleItems = 0;
	ControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	ControlDetails.paDetails = &aDetails[0];
	mixerGetControlDetails((HMIXEROBJ)m_dwMixerHandle, &ControlDetails, MIXER_GETCONTROLDETAILSF_VALUE);
	DWORD max = m_dwMinimalVolume;
	int i;
	for (i = 0; i < m_nChannelCount; i++)
	{
		if (aDetails[i].dwValue > max)
			max = aDetails[i].dwValue;
	}
	delete aDetails;
	max = 255 * (max - m_dwMinimalVolume) / (m_dwMaximalVolume - m_dwMinimalVolume);
	return max;
}
void CVolume::SetCurrentVolume(DWORD dwValue)
{
	if (!m_bAvailable)
		return;
	dwValue = (m_dwMaximalVolume - m_dwMinimalVolume) * dwValue / 255 + m_dwMinimalVolume;
	if (dwValue == 0)
		dwValue = (m_dwMaximalVolume - m_dwMinimalVolume) / 1000;
	MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
	if (!aDetails)
		return;
	for (int i = 0; i < m_nChannelCount; i++)
	{
		aDetails[i].dwValue = dwValue;
	}
	MIXERCONTROLDETAILS ControlDetails;

	memset(&ControlDetails, 0, sizeof(MIXERCONTROLDETAILS));
	ControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
	ControlDetails.dwControlID = m_dwVolumeControlID;
	ControlDetails.cChannels = m_nChannelCount;
	ControlDetails.cMultipleItems = 0;
	ControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	ControlDetails.paDetails = &aDetails[0];
	mixerSetControlDetails((HMIXEROBJ)m_dwMixerHandle, &ControlDetails, MIXER_GETCONTROLDETAILSF_VALUE);

	delete aDetails;
}

