/*****************************************************
** Module : CWinampWnd  							**
** Purpose : Winamp-style window					**
**  												**
** Copyright James Spibey, 2000 (spib@bigfoot.com)  **
** Feel free to use and distribute. May not be sold ** 
** for profit.  									**
*****************************************************/

#ifndef WINAMPWND_H
#define WINAMPWND_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ImageMap.h"
#include "BitmapEx.h"
#include "Resource.h"

// Winamp includes for General Plugin interface and IPC mechanism
#include <wa_ipc.h>

#define RESIZE_WIDTH		25
#define RESIZE_HEIGHT		29
#define MIN_WIDTH			275
#define MIN_HEIGHT			116
#define SNAP_OFFSET			10

#define DOCKED_TOP_LL		1	// Top Left Left
#define DOCKED_TOP_LR		2	// Top Left Right
#define DOCKED_TOP_RL		3	// Top Right Left
#define DOCKED_LEFT_TT		4
#define DOCKED_LEFT_TB		5
#define DOCKED_LEFT_BT		6
#define DOCKED_BOTTOM_LL	7
#define DOCKED_BOTTOM_RL	8
#define DOCKED_BOTTOM_LR	9
#define DOCKED_RIGHT_TT		10
#define DOCKED_RIGHT_TB		11
#define DOCKED_RIGHT_BT		12

class CWinampWnd : public CDialog
{
	// Construction
public:
	CWinampWnd(CWnd* pParent = NULL);   // standard constructor
	~CWinampWnd();

	BOOL Create(CWnd* pParent);
	void EnableDocking(BOOL bDock = TRUE)
	{
		m_bAllowDock = bDock;
	}
	void EnableResize(BOOL bResize = TRUE)
	{
		m_bAllowResize = bResize;
	}
	void SetWinampDir(CString Dir)
	{
		m_strWinampDir = Dir;
	};

	BOOL LoadSkin();

	// Called When Winamp Moves
	void TrackWindow(int x, int y);	
protected:
	void ChooseCursor(CPoint point);
	void DrawInterface(BOOL bPaint = FALSE);
	void Resize(CPoint point);
	void Dock(CPoint point = CPoint(0, 0));

	CBitmapEx m_Interface;				// Skin Bitmap
	CImageMap m_map;						// Imagemap

	BOOL m_bFocus;					// Do we have focus??

	// Resizing
	BOOL m_bAllowResize;				// Are we allowing resizing
	BOOL m_bResizing;				// Are we CURRENTLY resizing
	CPoint m_ptResizeFrom;				// Where did we start resizing from

	//Docking
	BOOL m_bDocked;					// Are we currently docked
	int m_nDockedSide;				// Which side are we docked to
	BOOL m_bAllowDock;				// Are we allowing docking
	int m_nSnapOffset;				// How far away should we snap to the window

	BOOL m_bDragging;				// Are we currently dragging
	CPoint m_ptDragFrom;				// Where did we start dragging

	int m_nHeightBeforeShadeMode;	// What were the window dimensions before shade mode

	CString m_strWinampDir;				// Path to Winamp Directory


	// Implementation
protected:

	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnShowWindow(BOOL bShow,UINT nStatus);
	afx_msg void OnPaint();
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnSetFocus( CWnd* pOldWnd );
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);

	DECLARE_MESSAGE_MAP()
};

#endif