// In_mpg123dParam.cpp
//
//////////////////////////////////////////////////////////////////////

#include "../stdafx.h"
#include "../SysTrayPlay.h"

#include "CommonFunc.h"

#include <wa_ipc.h>
#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "Param.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

CIn_mpg123dParam::CIn_mpg123dParam()
{
	// Initialize members

	m_nStartPage = 0;

	m_bEnable = true;
	m_nDecodeThreadPriority = 2 /* THREAD_PRIORITY_HIGHEST */;
	m_nOutputBps = 16;
	m_bSupZero = false;
	m_bDispAvg = false;
	m_bAltVolume = false;
	m_bReverseLR = false;
	m_bReversePhase = false;
	m_bFullBuffering = false;

	m_nVolume = 255;
	m_nPan = 0;

	m_bReplayGainEnable = false;
	m_nReplayGainMode = 0;
	m_nReplayGainTag = REPLAYGAIN_TAG_INIT;
	m_nReplayGainPreAmpWithRG = 0;
	m_nReplayGainPreAmpWithoutRG = 0;
	m_bReplayGainHardLimit = true;

	m_bEnableStreaming = false;
	m_nReceiveThreadPriority = 2;
	m_nBufferLength = 256;
	m_bProxy = false;
	m_strProxy = _T("");
	m_bSaveStream = false;
	m_strSaveStream = _T("");
	m_bTitleStreaming = true;
	m_bTitleStreamingDecode = true;
	m_bTitleStreamingUDP = false;
	m_strTitleStreamingFormat = _T("%1 (%2)");

	m_nTagPriority = 0;
	m_strID3Format = _T("%1 - %2");
	m_strSIFFormat = _T("%IART - %INAM");
}

CIn_mpg123dParam::~CIn_mpg123dParam()
{
	WriteProfile();
}

void CIn_mpg123dParam::GetProfile(void)
{
	static const WCHAR szAppName[] = _T("Shibatch mpg123");

	m_bEnable = theApp.GetProfileInt(szAppName, _T("Enable"), 1) != 0;
	m_nDecodeThreadPriority = theApp.GetProfileInt(szAppName, _T("DecodePriority"), 2);
	m_nOutputBps = theApp.GetProfileInt(szAppName, _T("OutputFormat"), 16);
	m_bFullBuffering = theApp.GetProfileInt(szAppName, _T("FullBuffering"), 0) != 0;
	m_bSupZero = theApp.GetProfileInt(szAppName, _T("SupZero"), 0) != 0;
	m_bReverseLR = theApp.GetProfileInt(szAppName, _T("ReverseLR"), 0) != 0;
	m_bReversePhase = theApp.GetProfileInt(szAppName, _T("ReversePhase"), 0) != 0;
	m_bAltVolume = theApp.GetProfileInt(szAppName, _T("Volume"), 0) != 0;
	m_bDispAvg = theApp.GetProfileInt(szAppName, _T("DispAvg"), 0) != 0;

	if (m_nOutputBps == 64)
		m_nOutputBps = -64;

	m_bPostProc = m_bReverseLR || m_bReversePhase || m_bAltVolume;

	m_nVolume = theApp.GetProfileInt(szAppName, _T("XVolume"), 255);
	m_nPan = theApp.GetProfileInt(szAppName, _T("XPan"), 0);

	m_bReplayGainEnable = theApp.GetProfileInt(szAppName, _T("ReplayGain_Enable"), 0) != 0;
	m_nReplayGainMode = theApp.GetProfileInt(szAppName, _T("ReplayGain_Mode"), 0);
	m_nReplayGainTag = theApp.GetProfileInt(szAppName, _T("ReplayGain_Tag"), REPLAYGAIN_TAG_INIT);
	m_nReplayGainPreAmpWithRG = theApp.GetProfileInt(szAppName, _T("ReplayGain_PreAmp_WithRG"), 0);
	m_nReplayGainPreAmpWithoutRG = theApp.GetProfileInt(szAppName, _T("ReplayGain_PreAmp_WithoutRG"), 0);
	m_bReplayGainHardLimit = theApp.GetProfileInt(szAppName, _T("ReplayGain_HardLimit"), 1) != 0;

	m_bEnableStreaming = theApp.GetProfileInt(szAppName, _T("EnableStreaming"), 0) != 0;
	m_nReceiveThreadPriority = theApp.GetProfileInt(szAppName, _T("ReceivePriority"), 2);
	m_nBufferLength = Max((int)GetProfileInt(szAppName, _T("StrBuf"), 256), MIN_STREAM_SIZE / 1024);
	m_bProxy = theApp.GetProfileInt(szAppName, _T("EnableProxy"), 0) != 0;
	m_bSaveStream = theApp.GetProfileInt(szAppName, _T("SaveStream"), 0) != 0;
	m_bTitleStreaming = theApp.GetProfileInt(szAppName, _T("TitleStreaming"), 1) != 0;
	m_bTitleStreamingUDP = theApp.GetProfileInt(szAppName, _T("UseUDP"), 0) != 0;

	m_strProxy = theApp.GetProfileString(szAppName, _T("Proxy"), _T(""));
	m_strSaveStream = theApp.GetProfileString(szAppName, _T("SaveStreamFile"), _T(""));
	m_strTitleStreamingFormat = theApp.GetProfileString(szAppName, _T("TitleStreamingFormat"), _T("%1 (%2)"));

	m_strID3Format = theApp.GetProfileString(szAppName, _T("FormatEx"), _T("[%artist% - ]%title%"));
	m_strSIFFormat = theApp.GetProfileString(szAppName, _T("SIFFormat"), _T("%IART - %INAM"));

	m_nTagPriority = theApp.GetProfileInt(szAppName, _T("TagPriority"), 0);
}

void CIn_mpg123dParam::WriteProfile(void)
{
	WCHAR string[32];
	static const WCHAR szAppName[] = _T("Shibatch mpg123");

	_itow(m_bEnable, string, 10);
	theApp.WriteProfileString(szAppName, _T("Enable"), string);

	_itow(m_nDecodeThreadPriority, string, 10);
	theApp.WriteProfileString(szAppName, _T("DecodePriority"), string);

	_itow(m_nOutputBps, string, 10);
	theApp.WriteProfileString(szAppName, _T("OutputFormat"), string);

	_itow(m_bFullBuffering, string, 10);
	theApp.WriteProfileString(szAppName, _T("FullBuffering"), string);

	_itow(m_bSupZero, string, 10);
	theApp.WriteProfileString(szAppName, _T("SupZero"), string);

	_itow(m_bReverseLR, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReverseLR"), string);

	_itow(m_bReversePhase, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReversePhase"), string);

	_itow(m_bAltVolume, string, 10);
	theApp.WriteProfileString(szAppName, _T("Volume"), string);

	_itow(m_bDispAvg, string, 10);
	theApp.WriteProfileString(szAppName, _T("DispAvg"), string);

	_itow(m_nVolume, string, 10);
	theApp.WriteProfileString(szAppName, _T("XVolume"), string);

	_itow(m_nPan, string, 10);
	theApp.WriteProfileString(szAppName, _T("XPan"), string);

	_itow(m_bReplayGainEnable, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_Enable"), string);

	_itow(m_nReplayGainMode, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_Mode"), string);

	_itow(m_nReplayGainTag, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_Tag"), string);

	_itow(m_nReplayGainPreAmpWithRG, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_PreAmp_WithRG"), string);

	_itow(m_nReplayGainPreAmpWithoutRG, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_PreAmp_WithoutRG"), string);

	_itow(m_bReplayGainHardLimit, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReplayGain_HardLimit"), string);

	_itow(m_bEnableStreaming, string, 10);
	theApp.WriteProfileString(szAppName, _T("EnableStreaming"), string);

	_itow(m_nReceiveThreadPriority, string, 10);
	theApp.WriteProfileString(szAppName, _T("ReceivePriority"), string);

	_itow(m_nBufferLength, string, 10);
	theApp.WriteProfileString(szAppName, _T("StrBuf"), string);

	_itow(m_bProxy, string, 10);
	theApp.WriteProfileString(szAppName, _T("EnableProxy"), string);

	_itow(m_bSaveStream, string, 10);
	theApp.WriteProfileString(szAppName, _T("SaveStream"), string);

	_itow(m_bTitleStreaming, string, 10);
	theApp.WriteProfileString(szAppName, _T("TitleStreaming"), string);

	_itow(m_bTitleStreamingDecode, string, 10);
	theApp.WriteProfileString(szAppName, _T("DecodeText"), string);

	_itow(m_bTitleStreamingUDP, string, 10);
	theApp.WriteProfileString(szAppName, _T("UseUDP"), string);

	theApp.WriteProfileString(szAppName, _T("Proxy"), (LPCTSTR)m_strProxy);
	theApp.WriteProfileString(szAppName, _T("SaveStreamFile"), (LPCTSTR)m_strSaveStream);
	theApp.WriteProfileString(szAppName, _T("TitleStreamingFormat"), (LPCTSTR)m_strTitleStreamingFormat);

	theApp.WriteProfileString(szAppName, _T("FormatEx"), (LPCTSTR)m_strID3Format);
	theApp.WriteProfileString(szAppName, _T("SIFFormat"), (LPCTSTR)m_strSIFFormat);

	_itow(m_nTagPriority, string, 10);
	theApp.WriteProfileString(szAppName, _T("TagPriority"), string);
}

WCHAR CIn_mpg123dParam::x2c(WCHAR* what)
{
	WCHAR digit;

	digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
	digit *= 16;
	digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A') + 10 : (what[1] - '0'));

	return digit;
}

void CIn_mpg123dParam::URLDecode(WCHAR* src, WCHAR* dst)
{
	WCHAR c;
	WCHAR* p;
	bool bFlag = false;
	bool bRadioSpy = false;

	p = src;

	while (c = (WCHAR)*p++)
	{
		switch (c)
		{
		case 0xff:
			if ((*p == 0x66) && (*(p + 1) == 0x66) && (*(p + 2) == 0x66) && (*(p + 3) == 0x66) && (*(p + 4)) && (*(p + 5)))
			{
				bRadioSpy = true;
				p += 4; /* 66 66 66 66 '8' 'C' */
				c = x2c(p);

				if (((c >= 0x81) && (c <= 0x9F)) || ((c >= 0xE0) && (c <= 0xEF)))
				{
					if (bFlag)
					{
						bFlag = false;
					}
					else
					{
						bFlag = true;
					}
				}
				else
				{
					bFlag = false;
				}

				p += 2;
			}
			else
			{
				if (bFlag)
				{
					*(dst - 1) = NULL;
				}
				bFlag = false;
				c = NULL;
			}
			break;
		case '%':
			bFlag = false;
			if ((*p) && (*(p + 1)))
			{
				c = x2c(p);
				p += 2;
			}
			else
			{
				c = NULL;
			}
			break;
		default:
			if ((c < 0x40) || (c > 0xfc))
			{
				if (bFlag)
				{
					dst--;
				}
			}

			if (bRadioSpy)
			{
				if ((c < 0x20) || (c >= 0x80))
				{
					*dst++ = NULL;
				}
			}

			bFlag = false;
		}
		*dst++ = c;
	}
	*dst = NULL;
}

int CIn_mpg123dParam::GetPriorityValue(int n)
{
	int ret;

	switch (n)
	{
	case 0:
		ret = THREAD_PRIORITY_NORMAL;
		break;
	case 1:
		ret = THREAD_PRIORITY_ABOVE_NORMAL;
		break;
	default:
		ret = THREAD_PRIORITY_HIGHEST;
		break;
	}

	return ret;
}

